@@ -31,6 +31,9 @@ import sys as _sys
 import os as _os
 import appdirs as _ad
 
+from threading import Lock
+mutex = Lock()
+
 try:
     import ujson as _json
 except ImportError:
@@ -316,34 +319,52 @@ class ProgressBar:
 
 
 # Simple file cache of ticker->timezone:
+_cache_dp = None
 def get_cache_dirpath():
-    return _os.path.join(_ad.user_cache_dir(), "py-yfinance")
+    if _cache_dp is None:
+        dp = _os.path.join(_ad.user_cache_dir(), "py-yfinance")
+    else:
+        dp = _os.path.join(_cache_dp, "py-yfinance")
+    return dp
+def set_tz_cache_location(dp):
+    global _cache_dp
+    _cache_dp = dp
+
 def cache_lookup_tkr_tz(tkr):
     fp = _os.path.join(get_cache_dirpath(), "tkr-tz.csv")
     if not _os.path.isfile(fp):
         return None
 
-    df = _pd.read_csv(fp)
-    f = df["Ticker"] == tkr
-    if sum(f) == 0:
+    mutex.acquire()
+    df = _pd.read_csv(fp, index_col="Ticker", on_bad_lines="skip")
+    mutex.release()
+    if tkr in df.index:
+        return df.loc[tkr,"Tz"]
+    else:
         return None
-
-    return df["Tz"][f].iloc[0]
 def cache_store_tkr_tz(tkr,tz):
-    df = _pd.DataFrame({"Ticker":[tkr], "Tz":[tz]})
 
     dp = get_cache_dirpath()
+    fp = _os.path.join(dp, "tkr-tz.csv")
+    mutex.acquire()
     if not _os.path.isdir(dp):
         _os.makedirs(dp)
-    fp = _os.path.join(dp, "tkr-tz.csv")
-    if not _os.path.isfile(fp):
-        df.to_csv(fp, index=False)
-        return
+    if (not _os.path.isfile(fp)) and (tz is not None):
+        df = _pd.DataFrame({"Tz":[tz]}, index=[tkr])
+        df.index.name = "Ticker"
+        df.to_csv(fp)
 
-    df_all = _pd.read_csv(fp)
-    f = df_all["Ticker"]==tkr
-    if sum(f) > 0:
-        raise Exception("Tkr {} tz already in cache".format(tkr))
-
-    _pd.concat([df_all,df]).to_csv(fp, index=False)
+    else:
+        df = _pd.read_csv(fp, index_col="Ticker", on_bad_lines="skip")
+        if tz is None:
+            # Delete if in cache:
+            if tkr in df.index:
+                df.drop(tkr).to_csv(fp)
+        else:
+            if tkr in df.index:
+                raise Exception("Tkr {} tz already in cache".format(tkr))
+            df.loc[tkr,"Tz"] = tz
+            df.to_csv(fp)
+    
+    mutex.release()
 
